/*
 * 
 */
package com.bankaudit.rest.controller;

import java.sql.Timestamp;
import java.util.List;

import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;

import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.bankaudit.constants.BankAuditConstant;
import com.bankaudit.dto.DataTableResponse;
import com.bankaudit.model.MaintAuditActivity;
import com.bankaudit.service.BusinessDateService;
import com.bankaudit.service.MaintAuditActivityService;
import com.bankaudit.service.SequenceAppenderService;
import com.bankaudit.util.BankAuditUtil;
import com.bankaudit.util.ServiceStatus;


/**
 * The Class {@link MaintAuditActivityController} provides the REST services to
 * manage the {@link MaintAuditActivity} class objects .
 *
 * @author amit.patel
 * @version 1.0
 */


@RestController

@RequestMapping("/api/maintAuditActivity")
public class MaintAuditActivityController {


	/**
	 * The maint audit activity service is autowired and make methods available
	 * from service layer .
	 */
	@Autowired
	MaintAuditActivityService maintAuditActivityService;
	
	/**
	 * The business date service is autowired and make methods available from
	 * service layer .
	 */
	@Autowired
	BusinessDateService businessDateService;
	
	/**
	 * The sequence appender service is autowired and make methods available
	 * from service layer .
	 */
	@Autowired 
	SequenceAppenderService sequenceAppenderService;
	
	/** The Constant logger is used to specify the . */
	static final Logger logger = Logger.getLogger(MaintAuditActivityController.class);
	
	/**
	 * This method is use to Creates the maint audit activity.
	 *
	 * @param maintAuditActivity
	 *            specify the maint audit activity
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(method=RequestMethod.POST,consumes=MediaType.APPLICATION_JSON_VALUE,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus createMaintAuditActivity(@RequestBody MaintAuditActivity maintAuditActivity){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		try {
			if(maintAuditActivity!=null
					&& maintAuditActivity.getLegalEntityCode()!=null
					&& !BankAuditUtil.isEmptyString(maintAuditActivity.getActivityCode())
					&& !BankAuditUtil.isEmptyString(maintAuditActivity.getActivityName())
					&& !BankAuditUtil.isEmptyString(maintAuditActivity.getMaker())
					){ 
				    maintAuditActivity.setActivityId(sequenceAppenderService.getAutoSequenceId());
				    maintAuditActivity.setBusinessdateTimestamp(businessDateService.getBusinessDateTimestampByLegalEntityCode(maintAuditActivity.getLegalEntityCode()));
				    maintAuditActivity.setSysdateTimestamp(new Timestamp(System.currentTimeMillis()));
				    maintAuditActivity.setMakerTimestamp(maintAuditActivity.getBusinessdateTimestamp());
				    
				    maintAuditActivityService.createMaintAuditActivity(maintAuditActivity);
				    
				    serviceStatus.setStatus("success");
				    serviceStatus.setMessage("successfully created");
			}else {
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("invalid payload ");
			}
		}catch (Exception e) {
			serviceStatus.setStatus("failure");
			if(e instanceof org.springframework.dao.DataIntegrityViolationException) {
				serviceStatus.setMessage("DATAINTGRTY_VIOLATION");
			}else serviceStatus.setMessage("EXCP_OCCURED");
		}
		
		return serviceStatus;
	}
	
	/**
	 * This method is use to Update maint audit activity.
	 *
	 * @param maintAuditActivity
	 *            specify the maint audit activity
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(method=RequestMethod.PUT,consumes=MediaType.APPLICATION_JSON_VALUE,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus updateMaintAuditActivity(@RequestBody MaintAuditActivity maintAuditActivity){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		if(maintAuditActivity!=null
				&& maintAuditActivity.getLegalEntityCode()!=null
				&& !BankAuditUtil.isEmptyString(maintAuditActivity.getActivityName())
				&& !BankAuditUtil.isEmptyString(maintAuditActivity.getActivityCode())
				&& !BankAuditUtil.isEmptyString(maintAuditActivity.getMaker())
				){
			
			try {
				
			    maintAuditActivity.setBusinessdateTimestamp(businessDateService.getBusinessDateTimestampByLegalEntityCode(maintAuditActivity.getLegalEntityCode()));
			    maintAuditActivity.setSysdateTimestamp(new Timestamp(System.currentTimeMillis()));
				if(maintAuditActivity.getStatus().equals(BankAuditConstant.STATUS_AUTH)){
					maintAuditActivity.setCheckerTimestamp(businessDateService.getBusinessDateTimestampByLegalEntityCode(maintAuditActivity.getLegalEntityCode()));
				} else{
					maintAuditActivity.setMakerTimestamp(businessDateService.getBusinessDateTimestampByLegalEntityCode(maintAuditActivity.getLegalEntityCode()));
				}  
			    maintAuditActivityService.updateMaintAuditActivity(maintAuditActivity);
			    
			    serviceStatus.setStatus("success");
			    serviceStatus.setMessage("successfully updated");
		
			} catch (Exception e) {
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
				if(e instanceof org.springframework.dao.DataIntegrityViolationException) {
					serviceStatus.setMessage("DATAINTGRTY_VIOLATION");
				}
			}
			
			
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload ");
		}
		
		return serviceStatus;
	}
	
	/**
	 * This method is use to Gets the maint audit activity.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param activityId
	 *            specify the activity id
	 * @param status
	 *            specify the status
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(value="/getByLegalEntityCodeAndActivityCode/{legalEntityCode}/{activityId}/{status}",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus getMaintAuditActivity(@PathVariable("legalEntityCode")Integer legalEntityCode,
			@PathVariable("activityId") String activityId,
			@PathVariable("status") String status
			){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		if(legalEntityCode!=null 
				&& !BankAuditUtil.isEmptyString(activityId)){
			
			try {
			    MaintAuditActivity maintAuditActivity=maintAuditActivityService.getMaintAuditActivity(legalEntityCode,activityId,status);
			    if(maintAuditActivity!=null){
			    	serviceStatus.setResult(maintAuditActivity);
				    serviceStatus.setStatus("success");
				    serviceStatus.setMessage("successfully retrieved ");	
			    }else {
					serviceStatus.setStatus("failure");
					serviceStatus.setMessage("maintAuditActivity not found");
				}
				
			} catch (Exception e) {
				e.printStackTrace();
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
			}
			
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload ");
		}
		
		return serviceStatus;
	}
	

	/**
	 * This method is use to Gets the maint audit activities.
	 *
	 * @param drawStr
	 *            specify the draw str
	 * @param length
	 *            specify the length
	 * @param start
	 *            specify the start
	 * @param search
	 *            specify the search
	 * @param orderColumnStr
	 *            specify the order column str
	 * @param orderDirection
	 *            specify the order direction
	 * @param legalEntityCodeStr
	 *            specify the legal entity code str
	 * @return the data table response class object with response status and
	 *         payload .
	 */
	
@RequestMapping(value="/getAll",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE) 
	DataTableResponse getMaintAuditActivities(
			@RequestParam("draw") String drawStr,
			@RequestParam("length") String length,
			@RequestParam("start") String start,
			@RequestParam("search[value]") String search,
			@RequestParam("order[0][column]") String orderColumnStr,
			@RequestParam("order[0][dir]")String orderDirection,
			@RequestParam("legalEntityCode")String legalEntityCodeStr
			){
		
		DataTableResponse dataTableResponse=null;
		
		if(!BankAuditUtil.isEmptyString(drawStr)
				&& !BankAuditUtil.isEmptyString(length)
				&& !BankAuditUtil.isEmptyString(start)
				&& !BankAuditUtil.isEmptyString(legalEntityCodeStr)){
			try {
				Integer legalEntityCode=Integer.parseInt(legalEntityCodeStr);
				Integer draw=Integer.parseInt(drawStr);
				Integer size=Integer.parseInt(length);
				Integer page=Integer.parseInt(start)/size;
				Integer orderColumn=null;
				if(!BankAuditUtil.isEmptyString(orderColumnStr)){
					orderColumn=Integer.parseInt(orderColumnStr);
				}
				dataTableResponse = maintAuditActivityService.getMaintAuditActivities(legalEntityCode,search,orderColumn,orderDirection,page,size);
				dataTableResponse.setDraw(draw);    
					
			} catch (Exception e) {
                dataTableResponse=new DataTableResponse();
                dataTableResponse.setError(e.getMessage());
				e.printStackTrace();
			}
		
		}else {
			dataTableResponse=new DataTableResponse();
			dataTableResponse.setError("invalid data");
		}
		
		return dataTableResponse;
	}
	
	/**
	 * This method is use to Gets the maint audit activity.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(value="/get/{legalEntityCode}",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus getMaintAuditActivity(@PathVariable("legalEntityCode")Integer legalEntityCode
			){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		if(legalEntityCode!=null 
				){
			
			try {
			    List<MaintAuditActivity> maintAuditActivities=maintAuditActivityService.getMaintAuditActivity(legalEntityCode);
			    if(maintAuditActivities!=null && !maintAuditActivities.isEmpty()){
			    	serviceStatus.setResult(maintAuditActivities);
				    serviceStatus.setStatus("success");
				    serviceStatus.setMessage("successfully retrieved ");	
			    }else {
					serviceStatus.setStatus("failure");
					serviceStatus.setMessage("maintAuditActivity not found");
				}
				
			} catch (Exception e) {
				e.printStackTrace();
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
			}
			
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload ");
		}
		
		return serviceStatus;
	}
	
	/**
	 * This method is use to Checks if is maint audit activity.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param activityCode
	 *            specify the activity code
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(value="/isMaintAuditActivity/{legalEntityCode}/{activityCode}",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus isMaintAuditActivity(@PathVariable("legalEntityCode")Integer legalEntityCode,
			@PathVariable("activityCode") String activityCode
			){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		if(legalEntityCode!=null 
				&& !BankAuditUtil.isEmptyString(activityCode)){
			
			try {
			    Boolean maintAuditActivity=maintAuditActivityService.isMaintAuditActivity(legalEntityCode,activityCode);
		    	serviceStatus.setResult(maintAuditActivity);
			    serviceStatus.setStatus("success");
			    serviceStatus.setMessage("successfully retrieved ");	
		    
				
			} catch (Exception e) {
				e.printStackTrace();
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
			}
			
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload ");
		}
		
		return serviceStatus;
	}
	
	// Service to delete the Records, 
	
@RequestMapping(value="/deleteActivityByID/{legalEntityCode}/{activityId}/{userId}",method=RequestMethod.DELETE,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus deleteActivityByID(@PathVariable("legalEntityCode")Integer legalEntityCode,
			@PathVariable("activityId") String activityId, @PathVariable("userId") String userId){
	ServiceStatus serviceStatus=new ServiceStatus();
		
		if(legalEntityCode !=null  && !BankAuditUtil.isEmptyString(activityId)){
			try {
				maintAuditActivityService.deleteActivityByID(legalEntityCode, activityId, userId);
				serviceStatus.setStatus("success");
				serviceStatus.setMessage("Record deleted successfully.");
			} catch (Exception e) {
				serviceStatus.setStatus("failure");
				e.printStackTrace();
				if(e instanceof org.springframework.dao.DataIntegrityViolationException) {
					serviceStatus.setMessage("DATAINTGRTY_VIOLATION");
				}
				serviceStatus.setMessage("failure");
			}
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload");
		}
		return serviceStatus;
	}	
	
}
