/*
 * 
 */
package com.bankaudit.rest.controller;

import java.sql.Timestamp;
import java.util.List;

import org.jboss.logging.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.bankaudit.constants.BankAuditConstant;
import com.bankaudit.dto.DataTableResponse;
import com.bankaudit.model.MaintAuditGroup;
import com.bankaudit.service.BusinessDateService;
import com.bankaudit.service.MaintAuditGroupService;
import com.bankaudit.service.SequenceAppenderService;
import com.bankaudit.util.BankAuditUtil;
import com.bankaudit.util.ServiceStatus;


/**
 * The Class {@link MaintAuditGroupController} provides the REST services to
 * manage the {@link MaintAuditGroup} class objects .
 *
 * @author amit.patel
 * @version 1.0
 */

@RestController

@RequestMapping("/api/maintAuditGroup")
public class MaintAuditGroupController {

	/**
	 * The maint audit group service is autowired and make methods available
	 * from service layer .
	 */
	@Autowired
	MaintAuditGroupService maintAuditGroupService;
	
	/**
	 * The business date service is autowired and make methods available from
	 * service layer .
	 */
	@Autowired
	BusinessDateService businessDateService;
	
	/**
	 * The sequence appender service is autowired and make methods available
	 * from service layer .
	 */
	@Autowired
	SequenceAppenderService sequenceAppenderService;
	
	/** The Constant logger is used to specify the . */
	static final Logger logger = Logger.getLogger(MaintAuditGroupController.class);
	
	/**
	 * This method is use to Creates the maint audit group.
	 *
	 * @param maintAuditGroup
	 *            specify the maint audit group
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(method=RequestMethod.POST,consumes=MediaType.APPLICATION_JSON_VALUE,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus createMaintAuditGroup(@RequestBody MaintAuditGroup maintAuditGroup){
		ServiceStatus serviceStatus=new ServiceStatus();
		try {
			if(maintAuditGroup!=null
					&& maintAuditGroup.getLegalEntityCode()!=null
					&& !BankAuditUtil.isEmptyString(maintAuditGroup.getAuditGroupCode())
					&& !BankAuditUtil.isEmptyString(maintAuditGroup.getAuditGroupName())
					&& !BankAuditUtil.isEmptyString(maintAuditGroup.getAuditTypeCode())
					&& !BankAuditUtil.isEmptyString(maintAuditGroup.getMaker())
					){ 
				
				maintAuditGroup.setBusinessdateTimestamp(businessDateService.getBusinessDateTimestampByLegalEntityCode(maintAuditGroup.getLegalEntityCode()));
				maintAuditGroup.setSysdateTimestamp(new Timestamp(System.currentTimeMillis()));
				maintAuditGroup.setMakerTimestamp(maintAuditGroup.getBusinessdateTimestamp());
				   
				maintAuditGroupService.createMaintAuditGroup(maintAuditGroup);
				serviceStatus.setStatus("success");
				serviceStatus.setMessage("successfully created");
			} else {
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("invalid payload");
			}
		}catch (Exception e) {
			serviceStatus.setStatus("failure");
			if(e instanceof org.springframework.dao.DataIntegrityViolationException) {
				serviceStatus.setMessage("DATAINTGRTY_VIOLATION");
			}else serviceStatus.setMessage("EXCP_OCCURED");
		}
		
		return serviceStatus;
	}
	
	/**
	 * This method is use to Update maint audit group.
	 *
	 * @param maintAuditGroup
	 *            specify the maint audit group
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(method=RequestMethod.PUT,consumes=MediaType.APPLICATION_JSON_VALUE,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus updateMaintAuditGroup(@RequestBody MaintAuditGroup maintAuditGroup){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		if(maintAuditGroup!=null
				&& maintAuditGroup.getLegalEntityCode()!=null
				&& !BankAuditUtil.isEmptyString(maintAuditGroup.getAuditGroupName())
				&& !BankAuditUtil.isEmptyString(maintAuditGroup.getAuditGroupCode())
				&& !BankAuditUtil.isEmptyString(maintAuditGroup.getAuditTypeCode())
				&& !BankAuditUtil.isEmptyString(maintAuditGroup.getMaker())
				){ 
		try{	
			maintAuditGroup.setBusinessdateTimestamp(businessDateService.getBusinessDateTimestampByLegalEntityCode(maintAuditGroup.getLegalEntityCode()));
			maintAuditGroup.setSysdateTimestamp(new Timestamp(System.currentTimeMillis()));
			if(maintAuditGroup.getStatus().equals(BankAuditConstant.STATUS_AUTH)){
				maintAuditGroup.setCheckerTimestamp(businessDateService.getBusinessDateTimestampByLegalEntityCode(maintAuditGroup.getLegalEntityCode()));
			}else{
				maintAuditGroup.setMakerTimestamp(businessDateService.getBusinessDateTimestampByLegalEntityCode(maintAuditGroup.getLegalEntityCode()));
			}  
			    maintAuditGroupService.updateMaintAuditGroup(maintAuditGroup);
			    
			    serviceStatus.setStatus("success");
			    serviceStatus.setMessage("successfully updated");
		} catch (Exception e) {
			e.printStackTrace();
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("failure");
			if(e instanceof org.springframework.dao.DataIntegrityViolationException) {
				serviceStatus.setMessage("DATAINTGRTY_VIOLATION");
			}
		}
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload ");
		}
		
		return serviceStatus;
	}
	
	/**
	 * This method is use to Gets the maint audit group.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param auditGroupCode
	 *            specify the audit group code
	 * @param auditTypeCode
	 *            specify the audit type code
	 * @param status
	 *            specify the status
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(value="/getByLegalEntityCodeAndAuditGroupCodeAndAuditTypeCode/{legalEntityCode}/{auditGroupCode}/{auditTypeCode}/{status}",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus getMaintAuditGroup(@PathVariable("legalEntityCode")Integer legalEntityCode,
			@PathVariable("auditGroupCode") String auditGroupCode,
			@PathVariable("auditTypeCode") String auditTypeCode,
			@PathVariable("status") String status
			){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		if(legalEntityCode!=null 
				&& !BankAuditUtil.isEmptyString(auditGroupCode)
				&& !BankAuditUtil.isEmptyString(auditTypeCode)){
			
			try {
			    MaintAuditGroup maintAuditGroup=maintAuditGroupService.getMaintAuditGroup(legalEntityCode,auditGroupCode,auditTypeCode,status);
			    if(maintAuditGroup!=null){
			    	serviceStatus.setResult(maintAuditGroup);
				    serviceStatus.setStatus("success");
				    serviceStatus.setMessage("successfully retrieved ");	
			    }else {
					serviceStatus.setStatus("failure");
					serviceStatus.setMessage("MaintAuditGroup not found");
				}
				
			} catch (Exception e) {
				e.printStackTrace();
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
			}
			
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload ");
		}
		
		return serviceStatus;
	}
	


	/**
	 * This method is use to Gets the maint audit groups.
	 *
	 * @param drawStr
	 *            specify the draw str
	 * @param length
	 *            specify the length
	 * @param start
	 *            specify the start
	 * @param search
	 *            specify the search
	 * @param orderColumnStr
	 *            specify the order column str
	 * @param orderDirection
	 *            specify the order direction
	 * @param legalEntityCodeStr
	 *            specify the legal entity code str
	 * @return the data table response class object with response status and
	 *         payload .
	 */
	
@RequestMapping(value="/getAll",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE) 
	DataTableResponse getMaintAuditGroups(
			@RequestParam("draw") String drawStr,
			@RequestParam("length") String length,
			@RequestParam("start") String start,
			@RequestParam("search[value]") String search,
			@RequestParam("order[0][column]") String orderColumnStr,
			@RequestParam("order[0][dir]")String orderDirection,
			@RequestParam("legalEntityCode")String legalEntityCodeStr){
		
		DataTableResponse dataTableResponse=null;
		
		if(!BankAuditUtil.isEmptyString(drawStr)
				&& !BankAuditUtil.isEmptyString(length)
				&& !BankAuditUtil.isEmptyString(start)
				&& !BankAuditUtil.isEmptyString(legalEntityCodeStr)){
			try {
				Integer legalEntityCode=Integer.parseInt(legalEntityCodeStr);
				Integer draw=Integer.parseInt(drawStr);
				Integer size=Integer.parseInt(length);
				Integer page=Integer.parseInt(start)/size;
				Integer orderColumn=null;
				if(!BankAuditUtil.isEmptyString(orderColumnStr)){
					orderColumn=Integer.parseInt(orderColumnStr);
				}
				dataTableResponse = maintAuditGroupService.getMaintAuditGroups(legalEntityCode,search,orderColumn,orderDirection,page,size);
				dataTableResponse.setDraw(draw);    
					
			} catch (Exception e) {
                dataTableResponse=new DataTableResponse();
                dataTableResponse.setError(e.getMessage());
				e.printStackTrace();
			}
		
		}else {
			dataTableResponse=new DataTableResponse();
			dataTableResponse.setError("invalid data");
		}
		
		return dataTableResponse;
	}
	
	/**
	 * This method is use to Gets the maint audit groups.
	 *
	 * @param legalEntityCodeStr
	 *            specify the legal entity code str
	 * @param auditTypeCode
	 *            specify the audit type code
	 * @param auditGroupCode
	 *            specify the audit group code
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(value="/get",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE) 
	ServiceStatus getMaintAuditGroups(
			@RequestParam("legalEntityCode")String legalEntityCodeStr,
			@RequestParam(required=false,value="auditTypeCode") String auditTypeCode,
			@RequestParam(required=false,value="auditGroupCode") String auditGroupCode
			){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		
		if(!BankAuditUtil.isEmptyString(legalEntityCodeStr)){
			try {
				Integer legalEntityCode=Integer.parseInt(legalEntityCodeStr);
				List<MaintAuditGroup> maintAuditGroups = maintAuditGroupService.getMaintAuditGroups(legalEntityCode,auditTypeCode,auditGroupCode);
				if(maintAuditGroups!=null && !maintAuditGroups.isEmpty()){
					serviceStatus.setStatus("success");
					serviceStatus.setMessage("successfuly retrieved");
					serviceStatus.setResult(maintAuditGroups);
				} else {
					serviceStatus.setStatus("failure");
					serviceStatus.setMessage("no records found");
				}
					
			} catch (Exception e) {
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
				e.printStackTrace();
			}
		
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("legalEntityCode required");
		}
		
		
		return serviceStatus;
	}
	
	/**
	 * This method is use to Checks if is maint audit group.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param auditTypeCode
	 *            specify the audit type code
	 * @param auditGroupCode
	 *            specify the audit group code
	 * @return the service status class object with response status and payload
	 *         .
	 */
	
@RequestMapping(value="/isMaintAuditGroup/{legalEntityCode}/{auditTypeCode}/{auditGroupCode}",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus isMaintAuditGroup(@PathVariable("legalEntityCode")Integer legalEntityCode,
			@PathVariable("auditTypeCode") String auditTypeCode,
			@PathVariable("auditGroupCode") String auditGroupCode
			){
		ServiceStatus serviceStatus=new ServiceStatus();
		
		if(legalEntityCode!=null 
				&& !BankAuditUtil.isEmptyString(auditTypeCode)){
			
			try {
				
			    Boolean maintAuditGroup=maintAuditGroupService.isMaintAuditGroup(legalEntityCode,auditTypeCode,auditGroupCode);
		    	serviceStatus.setResult(maintAuditGroup);
			    serviceStatus.setStatus("success");
			    serviceStatus.setMessage("successfully retrieved ");	
		    
				
			} catch (Exception e) {
				e.printStackTrace();
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
			}
			
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload ");
		}
		
		return serviceStatus;
	}
	
	
@RequestMapping(value="/isMaintAuditGroupLE/{legalEntityCode}/{auditGroupCode}",method=RequestMethod.GET,produces=MediaType.APPLICATION_JSON_VALUE)
	ServiceStatus isMaintAuditGroupLE(@PathVariable("legalEntityCode")Integer legalEntityCode,@PathVariable("auditGroupCode") String auditGroupCode	){
		ServiceStatus serviceStatus=new ServiceStatus();
		if(legalEntityCode!=null && !BankAuditUtil.isEmptyString(auditGroupCode)){
			try {
			    Boolean maintAuditGroupDuplicate=maintAuditGroupService.isMaintAuditGroupLE(legalEntityCode,auditGroupCode);
		    	serviceStatus.setResult(maintAuditGroupDuplicate);
			    serviceStatus.setStatus("success");
			    serviceStatus.setMessage("successfully retrieved ");	
			} catch (Exception e) {
				e.printStackTrace();
				serviceStatus.setStatus("failure");
				serviceStatus.setMessage("failure");
			}
			
		}else {
			serviceStatus.setStatus("failure");
			serviceStatus.setMessage("invalid payload ");
		}
		
		return serviceStatus;
	}
	
	// Service to delete the Records, 
			
@RequestMapping(value="/deleteAuditGroupByID/{legalEntityCode}/{auditTypeCode}/{auditGroupCode}/{userId}",method=RequestMethod.DELETE,produces=MediaType.APPLICATION_JSON_VALUE)
			ServiceStatus deleteAuditGroupByID(@PathVariable("legalEntityCode")Integer legalEntityCode,
					@PathVariable("auditTypeCode") String auditTypeCode, @PathVariable("auditGroupCode") String auditGroupCode,
					@PathVariable("userId") String userId){
			ServiceStatus serviceStatus=new ServiceStatus();
				
				if(legalEntityCode !=null && !BankAuditUtil.isEmptyString(auditTypeCode) && !BankAuditUtil.isEmptyString(auditGroupCode)){
					try {
						maintAuditGroupService.deleteAuditGroupByID(legalEntityCode, auditTypeCode, auditGroupCode, userId);
						serviceStatus.setStatus("success");
						serviceStatus.setMessage("Record deleted successfully.");
					} catch (Exception e) {
						serviceStatus.setStatus("failure");
						e.printStackTrace();
						if(e instanceof org.springframework.dao.DataIntegrityViolationException) {
							serviceStatus.setMessage("DATAINTGRTY_VIOLATION");
						}
						serviceStatus.setMessage("failure");
					}
				}else {
					serviceStatus.setStatus("failure");
					serviceStatus.setMessage("invalid payload");
				}
				return serviceStatus;
			}	

}
