/*
 * 
 */
package com.bankaudit.service;

import java.util.List;

import com.bankaudit.dto.DataTableResponse;
import com.bankaudit.model.MaintAuditGroup;

/**
 * The Interface {@link MaintAuditGroupService} define the methods that can be
 * exposed to controller layer in order to implements rest end-points and define
 * the business logic.
 *
 * @author amit.patel
 * @version 1.0
 */
public interface MaintAuditGroupService {

	/**
	 * This method is use to Creates the maint audit group.
	 *
	 * @param maintAuditGroup
	 *            specify the maint audit group
	 * @throws Exception
	 *             the exception
	 */
	void createMaintAuditGroup(MaintAuditGroup maintAuditGroup)throws Exception;

	/**
	 * This method is use to Update maint audit group.
	 *
	 * @param maintAuditGroup
	 *            specify the maint audit group
	 */
	void updateMaintAuditGroup(MaintAuditGroup maintAuditGroup);

	/**
	 * This method is use to Gets the maint audit group.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param auditGroupCode
	 *            specify the audit group code
	 * @param auditTypeCode
	 *            specify the audit type code
	 * @param status
	 *            specify the status
	 * @return the maint audit group .
	 */
	MaintAuditGroup getMaintAuditGroup(Integer legalEntityCode, String auditGroupCode,
			String auditTypeCode,String status);


	/**
	 * This method is use to Gets the maint audit groups.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param search
	 *            specify the search
	 * @param orderColumn
	 *            specify the order column
	 * @param orderDirection
	 *            specify the order direction
	 * @param page
	 *            specify the page
	 * @param size
	 *            specify the size
	 * @return the data table response .
	 */
	DataTableResponse getMaintAuditGroups(Integer legalEntityCode, String search, Integer orderColumn,
			String orderDirection, Integer page, Integer size);

	/**
	 * This method is use to Gets the maint audit groups.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param auditTypeCode
	 *            specify the audit type code
	 * @param auditGroupCode
	 *            specify the audit group code
	 * @return the list .
	 */
	List<MaintAuditGroup> getMaintAuditGroups(Integer legalEntityCode, String auditTypeCode, String auditGroupCode);

	/**
	 * This method is use to Checks if is maint audit group.
	 *
	 * @param legalEntityCode
	 *            specify the legal entity code
	 * @param auditTypeCode
	 *            specify the audit type code
	 * @param auditGroupCode
	 *            specify the audit group code
	 * @return the boolean .
	 */
	Boolean isMaintAuditGroup(Integer legalEntityCode, String auditTypeCode, String auditGroupCode);

	Boolean isMaintAuditGroupLE(Integer legalEntityCode, String auditGroupCode);
	
	void deleteAuditGroupByID(Integer legalEntityCode, String auditTypeCode, String auditGroupCode, String userId);
	
}
